@@ -1,11 +1,11 @@
 /****************************************************************************
 **
-** Copyright (C) 2019 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:GPL$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -14,14 +14,24 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
+**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,10 +55,12 @@
 #include <qhash.h>
 #include <qdebug.h>
 
-#include <unordered_map>
+#include <vector>
 
 QT_BEGIN_NAMESPACE
 
+static const QCalendarBackend *backendFromEnum(QCalendar::System system);
+
 namespace {
 
 struct CalendarName : public QString
@@ -61,7 +73,7 @@ inline bool operator==(const CalendarName &u, const CalendarName &v)
     return u.compare(v, Qt::CaseInsensitive) == 0;
 }
 
-inline uint qHash(const CalendarName &key, uint seed = 0) noexcept
+inline size_t qHash(const CalendarName &key, size_t seed = 0) noexcept
 {
     return qHash(key.toLower(), seed);
 }
@@ -110,19 +122,23 @@ struct Registry {
     }
     /*
         \internal
-        Ensures each enum-available calendar has been instantiated.
+        Ensures each \c{enum}-available calendar has been instantiated.
 
         This arranges for each to register itself by name; it only does anything on
         its first call, which ensures that name-based lookups can always find all
-        the calendars available via the enum.
+        the calendars available via the \c enum.
     */
     void populate()
     {
         if (populated)
             return;
 
-        for (int i = 0; i <= int(QCalendar::System::Last); ++i)
-            (void)QCalendar(QCalendar::System(i));
+        for (int i = 0; i <= int(QCalendar::System::Last); ++i) {
+            if (!byId[i])
+                (void)backendFromEnum(QCalendar::System(i));
+        }
+
+        populated = true;
     }
 };
 
@@ -130,6 +146,32 @@ struct Registry {
 
 Q_GLOBAL_STATIC(Registry, calendarRegistry);
 
+static const QCalendarBackend *backendFromEnum(QCalendar::System system)
+{
+    switch (system) {
+    case QCalendar::System::Gregorian:
+        return new QGregorianCalendar;
+#ifndef QT_BOOTSTRAPPED
+    case QCalendar::System::Julian:
+        return new QJulianCalendar;
+    case QCalendar::System::Milankovic:
+        return new QMilankovicCalendar;
+#endif
+#if QT_CONFIG(jalalicalendar)
+    case QCalendar::System::Jalali:
+        return new QJalaliCalendar;
+#endif
+#if QT_CONFIG(islamiccivilcalendar)
+    case QCalendar::System::IslamicCivil:
+        return new QIslamicCivilCalendar;
+#else // When highest-numbered system isn't enabled, ensure we have a case for Last:
+    case QCalendar::System::Last:
+#endif
+    case QCalendar::System::User:
+        Q_UNREACHABLE();
+    }
+    return nullptr;
+}
 
 /*!
     \since 5.14
@@ -156,8 +198,8 @@ Q_GLOBAL_STATIC(Registry, calendarRegistry);
 
     Most backends are pure code, with no data elements. Such backends should
     normally be implemented as singletons. For a backend to be added to the
-    QCalendar::System enum, it should be such a singleton, with a case in
-    QCalendar::fromEnum()'s switch statement to instantiate it.
+    QCalendar::System \c enum, it should be such a singleton, with a case in
+    backendFromEnum()'s switch statement (above) to instantiate it.
 
     Non-singleton calendar backends should ensure that each instance is created
     with a distinct primary name. Later instances attempting to register with a
@@ -189,9 +231,9 @@ QCalendarBackend::~QCalendarBackend()
 /*!
     The calendar system of this calendar.
 
-    Each calendar backend constructible from the QCalendar::System enum should
-    return the member of that enum that produces it. Other calendars should
-    return User.
+    Each calendar backend constructible from the QCalendar::System \c enum
+    should return the member of that \c enum that produces it. Other calendars
+    should return User.
 
     \sa QCalendarBackend::fromEnum()
 */
@@ -200,9 +242,16 @@ QCalendar::System QCalendarBackend::calendarSystem() const
     return QCalendar::System::User;
 }
 
+/*!
+    \fn QString QCalendarBackend::name() const;
+
+    This pure virtual method should be overloaded by each backend implementation
+    to return the name that the backend passes to the base-class as its name.
+*/
+
 /*!
     The primary name of this calendar.
- */
+*/
 QString QCalendar::name() const
 {
     return d ? d->name() : QString();
@@ -518,7 +567,7 @@ int QCalendarBackend::dayOfWeek(qint64 jd) const
 
 /*!
     \fn QString QCalendarBackend::dateTimeToString(QStringView format, const QDateTime &datetime,
-                                                   const QDate &dateOnly, const QTime &timeOnly,
+                                                   QDate dateOnly, QTime timeOnly,
                                                    const QLocale &locale) const
 
     Returns a string representing a given date, time or date-time.
@@ -574,6 +623,7 @@ bool QCalendarBackend::registerAlias(const QString &name)
 }
 
 /*!
+    \internal
     Returns a pointer to a named calendar backend.
 
     If the given \a name is present in availableCalendars(), the backend
@@ -581,7 +631,7 @@ bool QCalendarBackend::registerAlias(const QString &name)
     names ignores case. Note that this won't provoke construction of a calendar
     backend, it will only return ones that have been instantiated (and not yet
     destroyed) by some other means. However, calendars available via the
-    QCalendar::System enum are always registered when this is called.
+    QCalendar::System \c enum are always registered when this is called.
 
     \sa availableCalendars(), registerAlias(), fromEnum()
 */
@@ -595,6 +645,7 @@ const QCalendarBackend *QCalendarBackend::fromName(QStringView name)
 }
 
 /*!
+    \internal
     \overload
  */
 const QCalendarBackend *QCalendarBackend::fromName(QLatin1String name)
@@ -607,7 +658,8 @@ const QCalendarBackend *QCalendarBackend::fromName(QLatin1String name)
 }
 
 /*!
-    Returns a pointer to a calendar backend, specified by enum.
+    \internal
+    Returns a pointer to a calendar backend, specified by \c enum.
 
     This will instantiate the indicated calendar (which will enable fromName()
     to return it subsequently), but only for the Qt-supported calendars for
@@ -620,29 +672,7 @@ const QCalendarBackend *QCalendarBackend::fromEnum(QCalendar::System system)
     Q_ASSERT(calendarRegistry->byId.size() >= size_t(system));
     if (auto *c = calendarRegistry->byId[size_t(system)])
         return c;
-    switch (system) {
-    case QCalendar::System::Gregorian:
-        return new QGregorianCalendar;
-#ifndef QT_BOOTSTRAPPED
-    case QCalendar::System::Julian:
-        return new QJulianCalendar;
-    case QCalendar::System::Milankovic:
-        return new QMilankovicCalendar;
-#endif
-#if QT_CONFIG(jalalicalendar)
-    case QCalendar::System::Jalali:
-        return new QJalaliCalendar;
-#endif
-#if QT_CONFIG(islamiccivilcalendar)
-    case QCalendar::System::IslamicCivil:
-        return new QIslamicCivilCalendar;
-#else // When highest-numbered system isn't enabled, ensure we have a case for Last:
-    case QCalendar::System::Last:
-#endif
-    case QCalendar::System::User:
-        Q_UNREACHABLE();
-    }
-    return nullptr;
+    return backendFromEnum(system);
 }
 
 /*!
@@ -715,7 +745,10 @@ QCalendar::QCalendar()
 }
 
 QCalendar::QCalendar(QCalendar::System system)
-    : d(QCalendarBackend::fromEnum(system)) {}
+    : d(QCalendarBackend::fromEnum(system))
+{
+    Q_ASSERT(d);
+}
 
 QCalendar::QCalendar(QLatin1String name)
     : d(QCalendarBackend::fromName(name)) {}
@@ -750,6 +783,8 @@ int QCalendar::daysInMonth(int month, int year) const
 
 /*!
     Returns the number of days in the given \a year.
+
+    Handling of \c Unspecified as \a year is undefined.
 */
 int QCalendar::daysInYear(int year) const
 {
@@ -758,10 +793,15 @@ int QCalendar::daysInYear(int year) const
 
 /*!
     Returns the number of months in the given \a year.
+
+    If \a year is \c Unspecified, returns the maximum number of months in a
+    year.
+
+    \sa maximumMonthsInYear()
 */
 int QCalendar::monthsInYear(int year) const
 {
-    return d ? d->monthsInYear(year) : 0;
+    return d ? year == Unspecified ? d->maximumMonthsInYear() : d->monthsInYear(year) : 0;
 }
 
 /*!
@@ -1088,7 +1128,7 @@ QString QCalendar::standaloneWeekDayName(const QLocale &locale, int day,
     \sa monthName(), weekDayName(), QDate::toString(), QTime::toString()
 */
 QString QCalendar::dateTimeToString(QStringView format, const QDateTime &datetime,
-                                    const QDate &dateOnly, const QTime &timeOnly,
+                                    QDate dateOnly, QTime timeOnly,
                                     const QLocale &locale) const
 {
     return d ? d->dateTimeToString(format, datetime, dateOnly, timeOnly, locale) : QString();
