@@ -41,13 +41,17 @@
 #include "qsslcertificate_p.h"
 
 #include "qssl_p.h"
+#ifndef QT_NO_SSL
 #include "qsslkey.h"
 #include "qsslkey_p.h"
+#endif
 #include "qsslcertificateextension.h"
 #include "qsslcertificateextension_p.h"
 #include "qasn1element_p.h"
 
 #include <QtCore/qdatastream.h>
+#include <QtCore/qendian.h>
+#include <QtNetwork/qhostaddress.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -60,7 +64,7 @@ bool QSslCertificate::operator==(const QSslCertificate &other) const
     return d->derData == other.d->derData;
 }
 
-uint qHash(const QSslCertificate &key, uint seed) Q_DECL_NOTHROW
+size_t qHash(const QSslCertificate &key, size_t seed) noexcept
 {
     // DER is the native encoding here, so toDer() is just "return d->derData":
     return qHash(key.toDer(), seed);
@@ -137,14 +141,15 @@ QDateTime QSslCertificate::expiryDate() const
     return d->notValidAfter;
 }
 
-#ifndef Q_OS_WINRT // implemented in qsslcertificate_winrt.cpp
+#if !QT_CONFIG(schannel) // implemented in qsslcertificate_schannel.cpp
 Qt::HANDLE QSslCertificate::handle() const
 {
     Q_UNIMPLEMENTED();
-    return 0;
+    return nullptr;
 }
 #endif
 
+#ifndef QT_NO_SSL
 QSslKey QSslCertificate::publicKey() const
 {
     QSslKey key;
@@ -155,6 +160,7 @@ QSslKey QSslCertificate::publicKey() const
     }
     return key;
 }
+#endif
 
 QList<QSslCertificateExtension> QSslCertificate::extensions() const
 {
@@ -202,6 +208,10 @@ void QSslCertificatePrivate::init(const QByteArray &data, QSsl::EncodingFormat f
             : certificatesFromDer(data, 1);
         if (!certs.isEmpty()) {
             *this = *certs.first().d;
+#if QT_CONFIG(schannel)
+            if (certificateContext)
+                certificateContext = CertDuplicateCertificateContext(certificateContext);
+#endif
         }
     }
 }
@@ -336,10 +346,16 @@ bool QSslCertificatePrivate::parse(const QByteArray &data)
         return false;
 
     notValidBefore = elem.toDateTime();
+    if (!notValidBefore.isValid())
+        return false;
+
     if (!elem.read(validityStream) || (elem.type() != QAsn1Element::UtcTimeType && elem.type() != QAsn1Element::GeneralizedTimeType))
         return false;
 
     notValidAfter = elem.toDateTime();
+    if (!notValidAfter.isValid())
+        return false;
+
 
     // subject name
     if (!elem.read(certStream) || elem.type() != QAsn1Element::SequenceType)
@@ -386,23 +402,61 @@ bool QSslCertificatePrivate::parse(const QByteArray &data)
                     QSslCertificateExtension extension;
                     if (!parseExtension(elem.value(), &extension))
                         return false;
-                    extensions << extension;
 
                     if (extension.oid() == QLatin1String("2.5.29.17")) {
                         // subjectAltName
+
+                        // Note, parseExtension() returns true for this extensions,
+                        // but considers it to be unsupported and assignes a useless
+                        // value. OpenSSL also treats this extension as unsupported,
+                        // but properly creates a map with 'name' and 'value' taken
+                        // from the extension. We only support 'email', 'IP' and 'DNS',
+                        // but this is what our subjectAlternativeNames map can contain
+                        // anyway.
+                        QVariantMap extValue;
                         QAsn1Element sanElem;
                         if (sanElem.read(extension.value().toByteArray()) && sanElem.type() == QAsn1Element::SequenceType) {
                             QDataStream nameStream(sanElem.value());
                             QAsn1Element nameElem;
                             while (nameElem.read(nameStream)) {
-                                if (nameElem.type() == QAsn1Element::Rfc822NameType) {
+                                switch (nameElem.type()) {
+                                case QAsn1Element::Rfc822NameType:
                                     subjectAlternativeNames.insert(QSsl::EmailEntry, nameElem.toString());
-                                } else if (nameElem.type() == QAsn1Element::DnsNameType) {
+                                    extValue[QStringLiteral("email")] = nameElem.toString();
+                                    break;
+                                case QAsn1Element::DnsNameType:
                                     subjectAlternativeNames.insert(QSsl::DnsEntry, nameElem.toString());
+                                    extValue[QStringLiteral("DNS")] = nameElem.toString();
+                                    break;
+                                case QAsn1Element::IpAddressType: {
+                                    QHostAddress ipAddress;
+                                    QByteArray ipAddrValue = nameElem.value();
+                                    switch (ipAddrValue.length()) {
+                                    case 4: // IPv4
+                                        ipAddress = QHostAddress(qFromBigEndian(*reinterpret_cast<quint32 *>(ipAddrValue.data())));
+                                        break;
+                                    case 16: // IPv6
+                                        ipAddress = QHostAddress(reinterpret_cast<quint8 *>(ipAddrValue.data()));
+                                        break;
+                                    default: // Unknown IP address format
+                                        break;
+                                    }
+                                    if (!ipAddress.isNull()) {
+                                        subjectAlternativeNames.insert(QSsl::IpAddressEntry, ipAddress.toString());
+                                        extValue[QStringLiteral("IP")] = ipAddress.toString();
+                                    }
+                                    break;
+                                }
+                                default:
+                                    break;
                                 }
                             }
+                            extension.d->value = extValue;
+                            extension.d->supported = true;
                         }
                     }
+
+                    extensions << extension;
                 }
             }
         }
@@ -446,9 +500,9 @@ bool QSslCertificatePrivate::parseExtension(const QByteArray &data, QSslCertific
         if (!val.read(valElem.value()) || val.type() != QAsn1Element::SequenceType)
             return false;
         QVariantMap result;
-        const auto elems = val.toVector();
+        const auto elems = val.toList();
         for (const QAsn1Element &el : elems) {
-            QVector<QAsn1Element> items = el.toVector();
+            const auto items = el.toList();
             if (items.size() != 2)
                 return false;
             const QString key = QString::fromLatin1(items.at(0).toObjectName());
@@ -472,7 +526,7 @@ bool QSslCertificatePrivate::parseExtension(const QByteArray &data, QSslCertific
             return false;
 
         QVariantMap result;
-        QVector<QAsn1Element> items = val.toVector();
+        const auto items = val.toList();
         if (items.size() > 0) {
             result[QStringLiteral("ca")] = items.at(0).toBool(&ok);
             if (!ok)
@@ -491,7 +545,7 @@ bool QSslCertificatePrivate::parseExtension(const QByteArray &data, QSslCertific
         if (!val.read(valElem.value()) || val.type() != QAsn1Element::SequenceType)
             return false;
         QVariantMap result;
-        const auto elems = val.toVector();
+        const auto elems = val.toList();
         for (const QAsn1Element &el : elems) {
             if (el.type() == 0x80) {
                 const QString key = QStringLiteral("keyid");
