@@ -228,24 +228,24 @@ namespace QtAndroidInput
     static void touchAdd(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint id, jint action, jboolean /*primary*/, jint x, jint y,
         jfloat major, jfloat minor, jfloat rotation, jfloat pressure)
     {
-        Qt::TouchPointState state = Qt::TouchPointStationary;
+        QEventPoint::State state = QEventPoint::State::Stationary;
         switch (action) {
         case 0:
-            state = Qt::TouchPointPressed;
+            state = QEventPoint::State::Pressed;
             break;
         case 1:
-            state = Qt::TouchPointMoved;
+            state = QEventPoint::State::Updated;
             break;
         case 2:
-            state = Qt::TouchPointStationary;
+            state = QEventPoint::State::Stationary;
             break;
         case 3:
-            state = Qt::TouchPointReleased;
+            state = QEventPoint::State::Released;
             break;
         }
 
-        const int dw = desktopWidthPixels();
-        const int dh = desktopHeightPixels();
+        const int dw = availableWidthPixels();
+        const int dh = availableHeightPixels();
         QWindowSystemInterface::TouchPoint touchPoint;
         touchPoint.id = id;
         touchPoint.pressure = pressure;
@@ -258,7 +258,7 @@ namespace QtAndroidInput
                                  double(major * 2));
         m_touchPoints.push_back(touchPoint);
 
-        if (state == Qt::TouchPointPressed) {
+        if (state == QEventPoint::State::Pressed) {
             QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
             if (inputContext && qGuiApp)
                 QMetaObject::invokeMethod(inputContext, "touchDown", Q_ARG(int, x), Q_ARG(int, y));
@@ -275,15 +275,15 @@ namespace QtAndroidInput
         if (!platformIntegration)
             return;
 
-        QTouchDevice *touchDevice = platformIntegration->touchDevice();
+        QPointingDevice *touchDevice = platformIntegration->touchDevice();
         if (touchDevice == 0) {
-            touchDevice = new QTouchDevice;
-            touchDevice->setType(QTouchDevice::TouchScreen);
-            touchDevice->setCapabilities(QTouchDevice::Position
-                                         | QTouchDevice::Area
-                                         | QTouchDevice::Pressure
-                                         | QTouchDevice::NormalizedPosition);
-            QWindowSystemInterface::registerTouchDevice(touchDevice);
+            touchDevice = new QPointingDevice; // TODO fill out the constructor args
+            touchDevice->setType(QInputDevice::DeviceType::TouchScreen);
+            touchDevice->setCapabilities(QPointingDevice::Capability::Position
+                                         | QPointingDevice::Capability::Area
+                                         | QPointingDevice::Capability::Pressure
+                                         | QPointingDevice::Capability::NormalizedPosition);
+            QWindowSystemInterface::registerInputDevice(touchDevice);
             platformIntegration->setTouchDevice(touchDevice);
         }
 
@@ -343,7 +343,7 @@ namespace QtAndroidInput
 #endif
 
         QWindowSystemInterface::handleTabletEvent(tlw, ulong(time),
-            localPos, globalPosF, QTabletEvent::Stylus, pointerType,
+            localPos, globalPosF, int(QInputDevice::DeviceType::Stylus), pointerType,
             buttons, pressure, 0, 0, 0., 0., 0, deviceId, Qt::NoModifier);
 #endif // QT_CONFIG(tabletevent)
     }
